"""Data Integrity Fingerprint (DIF).

Reference Python implementation.

For specification see https://github.com/expyriment/DIF.

"""

__author__ = 'Oliver Lindemann <oliver@expyriment.org>, ' \
             'Florian Krause <florian@expyriment.org>'

__version__ = '0.7.3'


from .dif import DataIntegrityFingerprint
